/*
 
  LANChat Linux version 1.0.2 fully compatible with windows LC40
  Based on LC 4.0 specification by Piotr Kowalski <goro@vc.pl> 
  
  (c) Mateusz Skoczewski <melun@friko2.onet.pl>
  Antoby <antoby@mail.com>
 
  ----------------------------------------------------------------------
  The Linux LANChat and related utilities are free software; you can
  redistribute it and/or modify it under the terms of the GNU Library
  general Public License as published by the Free Software Foundation;
  either version 2 of the License, or (at your option) any later version.
 
*/
#include "lanchat.h"
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/in.h>

void usage(void)
{
    printf("Usage: LANChat [nickname] [-h host]\n\n");
    exit(1);
}

void cleanup(void)
{
    standend();
    curs_set(1);
    endwin();
    printf("\n");
}

void updatewin(WINDOW *win)
{
    wnoutrefresh( win );
    wdeleteln(inputwin); /* very bad way but works for now :) */
    waddstr(inputwin, readbuffer);
    wnoutrefresh( inputwin );
    doupdate();
}

void updateall(void)
{
    wnoutrefresh( headwin );
    wnoutrefresh( defwin );
    wnoutrefresh( userwin );
    wnoutrefresh( split1win );
    wnoutrefresh( split2win );
    wnoutrefresh( split3win );
    wnoutrefresh( inputwin );
    doupdate();
}

char *atime( int o )
{
    static char tt[ 9 ];
    static time_t now;
    static struct tm *tm;
    time( &now );
    tm = localtime( &now );
    strftime( tt, 9, "%H:%M:%S", tm );
    return tt;
}

char *adate(void)
{
    static char tt[ 12 ];
    static time_t now;
    static struct tm *tm;
    time( &now );
    tm = localtime( &now );
    strftime( tt, 12, "%d-%b-%Y", tm );
    return tt;
}

int auser( char *fuser )
{
    int num;
    for ( num = 0; num < 255; num++ )  if ( strcasecmp( users[ num ].nick, fuser ) == 0 ) return( num );
    return - 1;
}

void use_exit( int sig )
{
    cleanup();
    signal( SIGINT, use_exit );
}


int checkstr( char *ndata, char *nfuser )
{
    int i;
    char data[ 4096 ], fuser[ 20 ];
    strncpy( data, ndata, 4095 ); data[4095]='\0';
    strncpy( fuser, nfuser, 19 ); fuser[19]='\0';
    for ( i = 0; i <= strlen( data ); i++ ) data[ i ] = tolower( data[ i ] );
    for ( i = 0; i <= strlen( fuser ); i++ ) fuser[ i ] = tolower( fuser[ i ] );
    if ( strstr( data, fuser ) == 0 ) return 0;
    else return - 1;
}

void crypt( char *data, int cSender, int cDest ) 
{

    unsigned int qe, qq;
    int x;
    
    hsnd.xVal = rand() % 8 + 9;
    x = hsnd.xVal;
    qq = strlen( data );
    send_size = qq + 4;
    for ( qe = 0; qe < qq; qe++ ) 
        if ( data[ qe ] != 0 ){
             data[ qe ] = ( char ) data[ qe ] + ( 20-x );
             data[ qe ] = 255-data[ qe ];
             data[ qe ] = ( char ) ( data[ qe ] + cSender + cDest );
            }
    data[ qe + 1 ] = 'L';
    data[ qe + 2 ] = 'C';
    data[ qe + 3 ] = '4';
    data[ qe + 4 ] = '0';
}

void decrypt( int x, char *data, int cSender, int cDest ) 
{
    unsigned int qe, qq;

    qq = strlen( data );
    for ( qe = 0; qe < qq; qe++ ) 
	if ( data[ qe ] != 0 ){
	    data[ qe ] = ( char ) ( data[ qe ] - cSender - cDest );
            data[ qe ] = 255-data[ qe ];
            data[ qe ] = ( char ) ( data[ qe ] - 20 + x );
	}
}

void make_userlist( void )
{
    unsigned int i, j = 2;
    werase( userwin );
    wattrset( userwin, COLOR_PAIR( 5 ) | A_BOLD );
    waddstr( userwin, " ONLINE  LIST:\n\n" );
    for ( i = 0; i < 255; i++ ){
        if ( strlen( users[ i ].nick ) > 0 ){
            if ( users[ i ].away != 1 )
		wattrset( userwin, COLOR_PAIR( 4 ) | A_BOLD );
            else
		wattrset( userwin, COLOR_PAIR( 4 ) );
    	    mvwaddstr( userwin, j, 0, users[ i ].nick );
    	    wattrset( userwin, COLOR_PAIR( 4 ) );
    	    if ( users[ i ].away == 1 ) {
    	        mvwaddstr( userwin, j, 12, " (A)" ); /* less than 20 symbols for nick!!! */
    	    }
    	    j++;
	}
    }
    updatewin( userwin );
}

void decr( void )
{
    decrypt( hrcv.xVal, hrcv.data, hrcv.sender, hrcv.dest );
    hrcv.data[ strlen( hrcv.data ) - 2 ] = '\0';
}

void sline( void )
{
    wprintw( defwin, "%s\n", hrcv.data );
    updatewin( defwin );
}

void flog( char* a )
{
    chdir( "./logs" );
    f = fopen( adate(), "a" );
    fwrite( a, strlen( a ), 1, f );
    fclose( f );
    fflush( NULL );
}

void logtime(void)
{
    static char timestr[12];
    strcpy(timestr, "[");
    strcat(timestr, atime(0));
    strcat(timestr,"] ");
    flog(timestr);
}

void show_normal( void )
{
    int ncolor;
	
    if(hrcv.sender==hsnd.sender)
	ncolor = 3;
    else
	ncolor = 4;

    decr();

    wattrset( defwin, COLOR_PAIR( ncolor ) | A_BOLD );
    wprintw( defwin, "<" );

    if ( checkstr( hrcv.data, users[ hsnd.sender ].nick ) == 0 )
        wattrset( defwin, COLOR_PAIR( 5 ) );
    else
        wattrset( defwin, COLOR_PAIR( 8 ) | A_BOLD );

    wprintw( defwin, "%s", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( ncolor ) | A_BOLD );
    wprintw( defwin, "> " );
    wattrset( defwin, COLOR_PAIR( 5 ) );
    sline();

    logtime();
    flog( "<" );
    flog( hrcv.nick );
    flog( ">\t" );
    flog( hrcv.data );
    flog( "\n" );
}

void show_status( void )
{
    unsigned int i = 0;
    decr();

    waddstr( defwin, "* " );
    wattrset( defwin, A_BOLD );
    waddstr( defwin, "Nick: " );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;
    wattrset( defwin, A_BOLD );
    waddstr( defwin, " Address: " );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;
    wattrset( defwin, A_BOLD );
    waddstr( defwin, " IP=" );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;
    waddch( defwin, '\n' );
    wattrset( defwin, A_BOLD );
    waddstr( defwin, "Using: " );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;
    wattrset( defwin, A_BOLD );
    waddstr( defwin, " Under: " );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;

    wattrset( defwin, A_BOLD );
    waddstr( defwin, " User is: " );
    wattrset( defwin, A_NORMAL );
    if ( hrcv.data[ i ] == '0' ) waddstr( defwin, "alive" );
    else waddstr( defwin, "away" );
    i = i + 2;
    wattrset( defwin, A_BOLD );
    waddstr( defwin, " Online since: " );
    wattrset( defwin, A_NORMAL );
    while ( hrcv.data[ i ] != '\x07' ) {
        waddch( defwin, hrcv.data[ i ] );
        i++;
    }
    i++;
    waddch( defwin, '\n' );
    updatewin( defwin );
}

void show_me( void )
{
    if ( hrcv.dest != LC_BROADCAST ) {
        wattrset( defwin, COLOR_PAIR( 7 ) );
        wprintw( defwin, "msg" );
        wattrset( defwin, COLOR_PAIR( 4 ) );
        wprintw( defwin, "(" );
        wattrset( defwin, COLOR_PAIR( 5 ) );
        wprintw( defwin, "%s", hrcv.nick );
        wattrset( defwin, COLOR_PAIR( 4 ) );
        wprintw( defwin, ") " );
        wattrset( defwin, COLOR_PAIR( 5 ) );
    }
    wattrset( defwin, COLOR_PAIR( 6 ) | A_BOLD );
    wprintw( defwin, "* %s ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 5 ) | A_NORMAL );
    decr();
    sline();

    if ( hrcv.dest == LC_BROADCAST ) {
	logtime();
        flog( "* " );
        flog( hrcv.nick );
        flog( " " );
        flog( hrcv.data );
        flog( "\n" );
    }
}

void show_away( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "* " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "is away: " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    decr();
    sline();
    wattrset( defwin, COLOR_PAIR( 5 ) );
}

void show_pop( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "Popup from " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s: ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 8 ) | A_BOLD );
    decr();
    sline();
    beep();
    wattrset( defwin, COLOR_PAIR( 5 ) | A_BOLD );
    
    logtime();
    flog("Popup from ");
    flog(hrcv.nick);
    flog(": ");
    flog(hrcv.data);
    flog( "\n" );
	
}

void show_msg( void )
{
    wattrset( defwin, COLOR_PAIR( 7 ) );
    wprintw( defwin, "msg" );
    wattrset( defwin, COLOR_PAIR( 4 ) );
    wprintw( defwin, "(" );
    wattrset( defwin, COLOR_PAIR( 5 ) );
    wprintw( defwin, "%s", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 4 ) );
    wprintw( defwin, ") " );
    wattrset( defwin, COLOR_PAIR( 5 ) );
    decr();
    sline();

    logtime();
    flog("PrivMsg from ");
    flog(hrcv.nick);
    flog(": ");
    flog(hrcv.data);
    flog( "\n" );
}

void show_join( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "* joins ");
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, " " );
    decr();
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    waddstr( defwin, hrcv.data );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, " at " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", atime( 0 ) );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, ".\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    updatewin( defwin );

    logtime();
    flog( "* joins ");
    flog( hrcv.nick );
    flog( " " );
    flog( hrcv.data );
    flog( "\n" );
}

void change_topic( void )
{
    werase( headwin );
    decr();
    if ( !strcmp( hrcv.data, "<>" ) ) {
        wattrset( headwin, A_BOLD );
        wprintw( headwin, "LANChat v%s by Melun & Antoby", VER );
    }
    else {
        wattrset( headwin, A_BOLD );
        waddstr( headwin, "LANChat: " );
        wattrset( headwin, A_NORMAL );
        waddstr( headwin, hrcv.data );

    }
    if ( hrcv.type == LC_NEWTOPIC ) {
        wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
        wprintw( defwin, "* " );
        wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
        wprintw( defwin, "%s ", hrcv.nick );
        wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
        wprintw( defwin, "changed topic to: " );
        wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
        wprintw( defwin, "%s\n", hrcv.data );
        wattrset( defwin, COLOR_PAIR( 5 ) );
    }
    strncpy( topic, hrcv.data, 74); topic[74]='\0';
    wnoutrefresh( headwin );
    updatewin( defwin );

    logtime();
    flog("* ");
    flog(hrcv.nick);
    flog(" changes topic to: ");
    flog(hrcv.data);
    flog( "\n" );
}

void show_leave( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wmove(defwin, defwin->_cury, 0);
    wprintw( defwin, "* quits ");
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "at " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", atime( 0 ) );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, ".\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    updatewin( defwin );

    if ( hrcv.sender != hsnd.sender ) {
        logtime();
	flog( "* quits ");
        flog( hrcv.nick );
        flog( "\n" );
    }
}

void show_stats( void )  /* Eh, statistics should be better... */
{
    wprintw( defwin, "LANChat Session Statistics:\n" );
    if ( !cmp( topic, "<>" ) ) wprintw( defwin, "topic: there is no topic\n" );
    else {
        wprintw( defwin, "topic: %s\n", topic );
        wprintw( defwin, "since: %s\n", topictime );
    }

    wprintw( defwin, "Bytes received: %d\n", bytes_rcv );
    wprintw( defwin, "%d packets received\n", total_msg );

    updatewin( defwin );
}

void show_help( void )
{

    /* Who needs this colors anyway... all help'll be gray:) */
    wprintw( defwin, "--------==LANChat Commands Summary==-------\n" );
    wprintw( defwin, "/away <reason> - marks you as \"away\"\n" );
    wprintw( defwin, "/back - returns from \"away\" mode\n");
    wprintw( defwin, "/whois nickname - shows information about nick specified\n" );
    wprintw( defwin, "/topic <newtopic> - changes the topic\n" );
    wprintw( defwin, "/sa - show users away reasons\n" );
    wprintw( defwin, "/status - show some usage statistics\n" );
    wprintw( defwin, "/msg <nickname>, <body> - sends a private message\n" );
    wprintw( defwin, "/pop <nickname>, <body> - sends a pop-message\n" );
    wprintw( defwin, "/nick <new-nick> - changes your nickname\n" );
    wprintw( defwin, "/me <body> - send a description of what you're doing\n" );
    wprintw( defwin, "/clear or /cls - clears screen without sending mother-attack message\n" );
    wprintw( defwin, "/mother - clears screen and sends mother-attack message\n" );
    wprintw( defwin, "/credits - show some credits\n" );
    wprintw( defwin, "/help - what this can be...\n" );
    wprintw( defwin, "/quit - close LANChat session\n" );
    wprintw( defwin, "For more information about each command refer to README file\n" );

    updatewin( defwin );
}

void show_credits( void )
{
    wprintw( defwin, "LANChat Credits:\n" );
    wprintw( defwin, "Linux LANChat by melun <melun@friko2.onet.pl>.\n" );
    wprintw( defwin, "Lots of bugfixes & improvements by Antoby <antoby@mail.com>.\n" );
    wprintw( defwin, "Thanks to GORO for network and process code support.\n" );
    wprintw( defwin, "Greets to ABGC network, AJC network, Mi-V, BLADE, [:-{\n" );
    wprintw( defwin, "Thanks to all beta-testers and bug notifiers.\n" );
    wprintw( defwin, "Send your comments to lanchat@topnet.pl\n" );
    wprintw( defwin, "Visit http://www.lanchat.topnet.pl\n" );
    wprintw( defwin, "Enjoy !\n" );

    updatewin( defwin );
}

void show_resurrected( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "* " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "resurrected at " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", atime( 0 ) );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, ".\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    updatewin( defwin );
}

void show_unknown( void )
{
    decr();
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "Unknown command: " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", hrcv.data );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, " or command requires parameters\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    updatewin( defwin );
}

void show_went_away( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "* " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s ", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "went away at " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", atime( 0 ) );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, ": " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    decr();
    sline();
    wattrset( defwin, COLOR_PAIR( 5 ) );

    if ( hrcv.sender == hsnd.sender ) {
        snprintf( away, 249, "%s (%s)", hrcv.data, atime( 0 ) );
	away[249]='\0';
    }
    logtime();
    flog( "* ");
    flog( hrcv.nick );
    flog( " went away: " );
    flog( hrcv.data );
    flog( "\n" );
}

void show_back_away( void )
{
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "* " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", hrcv.nick );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, " came back at " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "%s", atime( 0 ) );
    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
    wprintw( defwin, "!\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    updatewin( defwin );

    logtime();
    flog( "* ");
    flog( hrcv.nick );
    flog( " came back!\n" );
}

int prepare_message( void )
{
    int i, nl = -1, nk = 0, np = 0;
    char tnick[ 20 ];

    decrypt( hrcv.xVal, hrcv.data, hrcv.sender, hrcv.dest );

    for (i=0; i<20; i++)
	if ( hrcv.data[ i ] == ',' ){
    	    if ( np == 0 )
		nk = i;
    	    np = 1;
	}
    if ( nk != 0 ) {
        nk--;
        strncpy( tnick, hrcv.data, nk );
        tnick[ nk ] = '\0';
        for ( i = 0; i < 255; i++ )
            if(!cmp( users[ i ].nick, tnick))
		nl = i;
        if (nl == -1){
    	    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
            wprintw( defwin, "error: user \"" );
            wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
            wprintw( defwin, "%s", tnick );
            wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
            wprintw( defwin, "\" not found.\n" );
            wattrset( defwin, COLOR_PAIR( 5 ) );
            updatewin( defwin );
        }else{
            hsnd.dest = nl;
            strncpy( hsnd.data, hrcv.data, 4095 ); hsnd.data[4095]='\0';
            strncpy( hsnd.data, hsnd.data + strlen( users[ nl ].nick ) + 2, strlen( hsnd.data ) - strlen( users[ nl ].nick ) + 2 );
        };
    }else{
        wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
        waddstr( defwin, "Usage: /command <nick name>, <message>\n" );
        waddstr( defwin, "You have to put ',' right after the nickname.\n" );
        wattrset( defwin, COLOR_PAIR( 5 ) );
        updatewin( defwin );
    }

    if ( nl == -1 ) return - 1;
    else return 0;
}

/* end lc_funcs.c */
